#include <stdio.h>
#include <math.h>
#define NMAX 64

//
// solve the initial value problem of an ordinary differential equation
// dy/dx = f(x, y) ( xmin <= x <= xmax ), y(xmin) = y0
// by the Euler method
//
// (usage)
// func(x, y) : the function f(x, y)
// y_init : the initial value y0
// [xmin, xmax] : the interval where the ordinary differential equation is solved
// nmax : the number of nodes
// xn[.] : the nodes x(n) ( n = 0, 1, ..., nmax ) will be stored in this array 
// yn[.] : the approximate values of y(x(n)) ( n = 0, 1, ..., nmax ) will be stored in this array
//
void euler_method(double (*func)(double, double), double y_init, 
		  double xmin, double xmax,
		  int nmax, double xn[NMAX+1], double yn[NMAX+1])
{
  int n;
  double h = (xmax - xmin) / nmax;   // h : the mesh
  //
  for (n=0; n<=nmax; ++n) xn[n] = xmin + n * h;
  //
  // step of the Euler method  
  //
  yn[0] = y_init;
  for (n=0; n<=nmax-1; ++n)
    yn[n+1] = yn[n] + h * func(xn[n], yn[n]);
}

double func(double x, double y)
{
  return y;
}

main()
{
  double xmin, xmax, y_init, xn[NMAX+1], yn[NMAX+1];
  int nmax, n;
  //
  y_init = 1.0;
  xmin = 0.0;
  xmax = 1.0;
  nmax = 16;
  euler_method(func, y_init, xmin, xmax, nmax, xn, yn);
  //
  printf("#    x(n)         y(n)     \n");
  printf("#--------------------------\n");
  for (n=0; n<=nmax; ++n)
    printf("%12.5e %12.5e\n", xn[n], yn[n]);
  //
  return 0;
}
